package scorekeeping;

import static gui.UpopVote.MAX_TEAMS;
import static gui.UpopVote.MAX_TEAM_SIZE;
import gui.GameFlowModel;
import gui.Team;
import gui.UpopVote;

import java.util.Map;
import java.util.Vector;

/**
 * Question object contains the text of a question, correct answers,
 * strange answers, and wrong answers.  In addition, each question
 * stores all votes received for the particular question. Question
 * objects represent a particular state of the game.
 *
 * @author Michael Gebauer
 */
public class Question {

  public static enum AnswerType {correct,strange,wrong};

  // About the question
  private String question;
  private Map<String,AnswerType> answers;

  // about the votes
  private Vote[][] votes = new Vote[MAX_TEAMS][MAX_TEAM_SIZE];
  private boolean[] scoreAlreadyReturned = new boolean[MAX_TEAMS];

  // about the game
  private static boolean[][] present = new boolean[MAX_TEAMS][MAX_TEAM_SIZE];
  private static int[] totalPresent = new int[MAX_TEAMS];

  public Question(String question, Map<String,AnswerType> answers) {
    this.question = question;
    this.answers = answers;
  }

  public int getAnswerCount() {
    return answers.size();
  }

  public String getQuestion() {
    return question;
  }

  // Returns the stored votes.  Useful for summarizing votes.
  // In the returned 2D array, rows represent teams and columns
  // represent positions on each team.  For example, the vote in
  // row 3 (i.e. the row with index 2), and column 4 (i.e. column with
  // index 3) represents the vote stored for the 4th player
  // on team 3.
  public Vote[][] getVotes() {
    Vote[][] voteArray = new Vote[MAX_TEAMS][MAX_TEAM_SIZE];
    for (int i=0; i<MAX_TEAMS; i++) {
      for (int j=0; j<MAX_TEAM_SIZE; j++) {
        voteArray[i][j] = votes[i][j];
      }
    }
    return voteArray;
  }

  // returns true if the parameter corresponds to a permissible vote
  public boolean isAcceptable(int vote) {
    return vote<=answers.size() && vote>0;
  }

  //  Returns true iff the vote corresponds to a wrong answer.
  public boolean isWrongAnswer(int i) {
    Vector<AnswerType> types = new Vector<AnswerType>(answers.values());
    return types.elementAt(i)==AnswerType.wrong;
  }

  //  Returns true iff the vote corresponds to a strange answer.
  public boolean isStrangeAnswer(int i) {
    Vector<AnswerType> types = new Vector<AnswerType>(answers.values());
    return types.elementAt(i)==AnswerType.strange;
  }



  public boolean isVotingComplete() {
    boolean complete = true;
    for (int i=0; i<MAX_TEAMS; i++)
      if (!scoreAlreadyReturned[i] && totalPresent[i]>0)
        complete = false;
    return complete;
  }

  public String toString() {
    return toString(false);
  }

  // returns html
  public String toString(boolean asAnswer) {
    StringBuilder output = new StringBuilder("<p>"+question+"</p><ol>");
    for (String answer : answers.keySet()) {
      AnswerType type = answers.get(answer);
      if (asAnswer && type == AnswerType.wrong)
        output.append("<li color=\"RED\">"+answer+" (wrong)</li>");
      else if (asAnswer && type == AnswerType.strange)
        output.append("<li color=\"BLUE\">"+answer+" (strange)</li>");
      else
        output.append("<li color=\"BLACK\">"+answer+"</li>");
    }
    output.append("</ol>");
    return output.toString();
  }

  // Returns true iff everyone on the team has finished voting.
  public boolean receiveVote(Team t, Vote v) {
    int teamno = t.getNumber();
    int playerno = t.getPlayerNumber(v.getId());
    if (present[teamno][playerno] && !scoreAlreadyReturned[teamno])
      votes[teamno][playerno] = v;

    // check if everyone has voted
    boolean teamFinished = true;
    for (int i=0; i<MAX_TEAM_SIZE; i++)
      if (present[teamno][i] && (votes[teamno][i]==null))
        teamFinished = false;
    return teamFinished;
  }

  // Only returns the score for a team once, and returns 0 after.
  // This is because the question is the only object which internally
  // stores whether the question has score has been returned.
  // It is important to know whether the score has already been
  // returned because teams should not receive points multiple
  // times for the same question.  Admittedly, it might be more
  // clear to store this state information elsewhere in the
  // UpopVote system.
  public synchronized double getTeamScore(Team t, GameFlowModel.State state) {
    int teamno = t.getNumber();
    int[] countArray = new int[10];
    if (scoreAlreadyReturned[teamno]) {
      return 0;
    } else {
      // Calculate the maximum number of agreeing votes
      for (Vote v : votes[teamno]) {
        if (v!= null
            && v.getValue()!=0
            && !isWrongAnswer(v.getValue()-1)) {
          countArray[v.getValue()] += 1;
        }
      }

      double max = -1;
      int maxIndex = -1;
      for (int i=0; i<countArray.length; i++) {
        if (countArray[i]>max) {
          max = countArray[i];
          maxIndex = i;
        }
      }
      // if everyone answered correctly then give a 10 point bonus
      // don't give the bonus if the max answer was "strange"
      if (max == totalPresent[teamno] && max>0) {
          max += 10;
      }

      // double the score if in round 2 (and answer isn't strange)
      if (GameFlowModel.State.round_2 == state &&
          max>0 &&
          !isStrangeAnswer(maxIndex-1)) {
        max = max*2;
      }

      if (max>0 && isWrongAnswer(maxIndex-1)) {
        max = 0;
      }

      scoreAlreadyReturned[teamno] = true;
      return max;
    }
  }


  public static void registerPlayer(Team t, Vote v) {
    int teamno = t.getNumber();
    int playerID = t.getPlayerNumber(v.getId());

    // if the player wasn't previously registered, then increase
    // the number of players on that team
    if (!present[teamno][playerID])
      totalPresent[teamno] += 1;

    // register the player
    present[teamno][playerID] = true;

    UpopVote.print("Question: registered player: ","\t"+v,"\t"+t);
  }

  public String writeSummary() {
    StringBuffer summary = new StringBuffer();
    summary.append(this.toString(true)+"\n");
    for (Vote[] teamVotes: votes) {
      for (Vote vote: teamVotes)
        if (vote!=null)
          summary.append(vote.toString()+"\t");
        else
          summary.append(null+"\t");
      summary.append("\n");
    }
    return summary.toString();
  }
}

//  LocalWords:  iff ol li UpopVote
