package scorekeeping;

import gui.ScoreboardModel;
import gui.Team;
import gui.GameFlowModel;

/**
 * This class handles all incoming votes and sends them to the
 * appropriate destination, which is expected to be a Question
 * object.
 */
public class VoteReceiver {

  public static boolean verbose = false;
  // public static boolean verbose = true;
  private static Question destination = null;
  private static ScoreboardModel scoreboardModel = null;
  private static GameFlowModel timer = null;
  private static boolean isPaused = true;

  private static boolean registrationMode = true;

  /**
   * This method is made private because it should not be called.
   * All methods in this class are static.
   */
  private VoteReceiver() {
  }

  public static void setPaused(boolean pauseState) {
    isPaused = pauseState;
  }

  /**
   * Sets where votes should be directed once they are received.
   * Typically, this will be a Question object.  Set the destination
   * to null, if received votes should be disregarded.
   *
   * @par am newDestination the object which should receive votes.
   */
  public static synchronized void setDestination(Question newDestination) {
    // Update the scores for any teams which didn't receive votes
    // from all players.
    if (newDestination == null && destination!=null) {
      double teamScore;
      for (Team t : Team.values()) {
        teamScore = destination.getTeamScore(t,timer.getGameState());
        scoreboardModel.displayScoreUpdate(t,teamScore);
      }
    }
    // update the destination
    VoteReceiver.destination = newDestination;
  }

  /**
   * Sets the scoreboard model.  IDs will be removed from the
   * scoreboard model once votes for them are received if this is set.
   */
  public static void setScoreBoardModel(ScoreboardModel scoreboardModel) {
    VoteReceiver.scoreboardModel = scoreboardModel;
  }

  /**
   * Sets the timer.
   *
   * @param timer
   */
  public static void setTimer(GameFlowModel timer) {
    VoteReceiver.timer = timer;
  }

  /**
   * Sets the destination of received votes to null.
   */
  public static synchronized void stopReceiving() {
    destination = null;
  }

  /**
   * If true, votes will be printed when received.
   *
   * @param b
   */
  public static void setVerbose(boolean b) {
    verbose = b;
  }

  // This is the method in EZJCom.__ctlPrsX that this class must override.
  /**
   * Called when a vote is received.
   */
  public static synchronized void receive(short[] port, int[] id, short[] value, short[] confidence) {
    if (!((port.length == 1)
          && (id.length == 1)
          && (value.length == 1)
          && (confidence.length == 1))) {
      System.err.printf("Non-unity array lengths in Receive: %d %d %d %d%n",
                port.length, id.length, value.length, confidence.length);
    }
		// Sometimes the hardware delivers (many instances of) Vote[1,0,0,0].
		if (id[0] == 0) {
			// System.err.println("ID = 0: " + v);
			return;
		}

    if (isPaused) {
      return;
    }

    Vote v = new Vote(port[0], id[0], value[0], confidence[0]);
    Team t = Team.findTeam(id[0]);

    // if the destination is not null, compute the vote
    if (destination != null) {
      if (verbose) { System.err.println(v); }

      // turn off registration mode
      registrationMode = false;

      // ensure that the vote is permissible
      if (destination.isAcceptable(value[0])) {
        boolean teamFinished = destination.receiveVote(t,v);
        if (scoreboardModel != null) {
          // remove the id from the scoreboard and update the score
          // if the team has finished voting
          if (teamFinished) {
            double teamScore = destination.getTeamScore(t,timer.getGameState());
            scoreboardModel.displayScoreUpdate(t,teamScore);
            // if all teams are done, advance the time to 0
            if (destination.isVotingComplete() && timer!=null) {
              timer.next();
            }
          } else scoreboardModel.removeID(t,id[0]);
        }
      }
    } else if (registrationMode) {
      if (verbose) { System.err.println(v); }

      scoreboardModel.registerPlayer(t,v);
      Question.registerPlayer(t,v);
    }
  }
}

//  LocalWords:  newDestination param EZJCom ctlPrsX
